#!/usr/bin/env python

############################################################################
# ultimate_guitar.py
# (c) 2006 Jesus Serrano <jesus.serrano@gmail.com>
# 
# Find guitar tabs on http://www.ultimate-guitar.com/
# Version 0.0.3
#
############################################################################
# Python-Qt template script for amaroK
# (c) 2005 Mark Kretschmann <markey@web.de>
#
# Depends on: Python 2.2, PyQt
############################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
############################################################################

import ConfigParser
import os
import sys
import threading
import signal
import commands
from time import sleep


try:
    from qt import *
except:
    os.popen( "kdialog --sorry 'PyQt (Qt bindings for Python) is required for this script.'" )
    raise


debug_prefix = "UltimateGuitar"
#debug_file="ultimateguitar.log"
version="0.0.1"


class ConfigDialog:
    """ Configuration widget """

    def __init__( self ):
        self.command="mozilla-firefox"
        try: 
          conf=ConfigParser.ConfigParser()
          conf.read("ultimateguitarrc")
          self.command=conf.get("General","commandbrowser")         
        except:
            pass

    
        
    def config( self ):
        old_command=self.command
        debug("config: dialog")
        ret=os.popen('kdialog --inputbox "Set your command browser. Examples: firefox or konqueror" "%s"' % self.command)
        self.command=ret.readline().replace("\n","")        
        if self.command:
          self.save()
        else:
            debug("command blank not save")
            self.command=old_command

    
    def save( self ):
        debug("config: save")
        """ Saves configuration to file """

        self.file = file( "ultimateguitarrc", 'w' )

        self.config = ConfigParser.ConfigParser()
        self.config.add_section( "General" )        
        self.config.set( "General", "commandbrowser", self.command )
        self.config.write( self.file )
        self.file.close()


class Notification( QCustomEvent ):
    __super_init = QCustomEvent.__init__
    def __init__( self, str ):
        self.__super_init(QCustomEvent.User + 1)
        self.string = str

class UltimateGuitar( QApplication ):
    """ The main application, also sets up the Qt event loop """

    def __init__( self, args ):
        QApplication.__init__( self, args )
        debug( "Started." )

        self.bandmenu=""
        self.songmenu=""
        self.add_menu()

        # Start separate thread for reading data from stdin
        self.stdinReader = threading.Thread( target = self.readStdin )
        self.stdinReader.start()

        self.readSettings()

    def delete_menu(self):
        debug("delete_menu")
        if self.bandmenu:
            os.popen('dcop amarok script removeCustomMenuItem "Ultimate Guitar" "%s"' % self.bandmenu)
        if self.songmenu:
            os.popen('dcop amarok script removeCustomMenuItem "Ultimate Guitar" "%s"' % self.songmenu)

        

    
    def add_menu(self):
        debug("add_menu")
        song=self.get_song()
        if song:
            os.popen('dcop amarok script addCustomMenuItem "Ultimate Guitar" "%s"' % song)

        band = self.get_band()
        if band:
            os.popen('dcop amarok script addCustomMenuItem "Ultimate Guitar" "%s"' % band)


    def get_band(self):
        band=commands.getoutput("dcop amarok player artist")
        if band:
            debug("get_band band:"+band)
            global bandmenu
            bandmenu="Band:"+ band
            self.bandmenu=bandmenu
            return bandmenu
        return ""

    def get_song(self):
        song=commands.getoutput("dcop amarok player title")
        if song:
            debug("get_song song:"+song)
            global songmenu
            songmenu="Song: "+song
            self.songmenu=songmenu
            return songmenu
        return ""
            
    

    def readSettings( self ):        
        """ Reads settings from configuration file """
        debug("readSettings")
        try:            
            dia = ConfigDialog()
            self.command=dia.command
            debug("command: "+self.command)
        except:
            debug( "No config file found, using defaults." )


############################################################################
# Stdin-Reader Thread
############################################################################

    def readStdin( self ):
        """ Reads incoming notifications from stdin """

        while True:
            # Read data from stdin. Will block until data arrives.
            line = sys.stdin.readline()
            if line:
                if "customMenuClicked" in line:                
                
                    if "Song:" in line:
                        song = commands.getoutput("dcop amarok player title")
			song = song.replace(' ', '%20')
                        url="http://www.ultimate-guitar.com/search.php?s=%s&w=songs" % song
                    elif "Band:" in line:
                        band = commands.getoutput("dcop amarok player artist")
			band = band.replace(' ', '%20')
                        url="http://www.ultimate-guitar.com/search.php?s=%s&w=bands" % band
                    else:
                        pass
                        
                    os.system("%s '%s' &" % (self.command,url))
                qApp.postEvent( self, Notification(line) )
            else:
                break


############################################################################
# Notification Handling
############################################################################

    def customEvent( self, notification ):
        """ Handles notifications """

        string = QString(notification.string)
        #debug( "Received notification: " + str( string ) )

        if string.contains( "configure" ):
            self.configure()

        if string.contains( "engineStateChange: playing" ):
            self.engineStatePlay()

        if string.contains( "engineStateChange: idle" ):
            self.engineStateIdle()

        if string.contains( "engineStateChange: pause" ):
            self.engineStatePause()

        if string.contains( "engineStateChange: empty" ):
            self.engineStatePause()

        if string.contains( "trackChange" ):
            self.trackChange()


# Notification callbacks. Implement these functions to react to specific notification
# events from amaroK:

    def configure( self ):
        debug( "configuration" )

        self.dia = ConfigDialog()
        self.dia.config()

        self.command=self.dia.command
        debug("new_command:"+self.command)

    def engineStatePlay( self ):
        """ Called when Engine state changes to Play """
        debug("engineStatePlay")
        self.delete_menu()
        self.add_menu()
        pass

    def engineStateIdle( self ):
        debug("engineStateIdle")
        """ Called when Engine state changes to Idle """
        pass

    def engineStatePause( self ):
        debug("engineStatePause")
        self.delete_menu()
        """ Called when Engine state changes to Pause """
        pass

    def engineStateEmpty( self ):
        debug("engineStateEmpty")
        """ Called when Engine state changes to Empty """
        pass

    def trackChange( self ):
        """ Called when a new track starts """        
        debug("trackChange")
        self.delete_menu()
        self.add_menu()

        pass


    
############################################################################

def debug( message ):
    """ Prints debug message to stdout """
    print debug_prefix + " " + message
    try:
        if debug_file:
            f=file(debug_file,"a")
            f.write(debug_prefix + " " + message + "\n")
    except:
	pass # no logs

def main( ):
    app = UltimateGuitar( sys.argv )

    app.exec_loop()

def onStop(signum, stackframe):    
    """ Called when script is stopped by user """
    debug("Stop")
    os.popen('dcop amarok script removeCustomMenuItem "Ultimate Guitar" "%s"' % bandmenu)
    os.popen('dcop amarok script removeCustomMenuItem "Ultimate Guitar" "%s"' % songmenu)
    
    pass

if __name__ == "__main__":
    mainapp = threading.Thread(target=main)
    mainapp.start()
    signal.signal(15, onStop)

    # necessary for signal catching
    while 1: sleep(120)

